# Generated by Django 4.1.3 on 2023-01-02 02:34

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("formkit_ninja", "0001_initial"),
    ]

    operations = [
        # migrations.RemoveField(
        #     model_name="formkitschemanode",
        #     name="children",
        # ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="else_condition",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="html_id",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="if_condition",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="key",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="name",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="node_value",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="placeholder",
        ),
        migrations.RemoveField(
            model_name="formkitschemanode",
            name="then_condition",
        ),
        migrations.AddField(
            model_name="formkitschemanode",
            name="values",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="formkitschemanode",
            name="node_type",
            field=models.CharField(
                blank=True,
                choices=[
                    ("$cmp", "Component"),
                    ("text", "Text"),
                    ("condition", "Condition"),
                    ("$formkit", "FormKit"),
                    ("$el", "Element"),
                ],
                max_length=256,
            ),
        ),
    ]
