# Generated by Django 4.1.3 on 2023-01-03 13:35

from django.db import migrations

import formkit_ninja.models


class Migration(migrations.Migration):

    dependencies = [
        ("formkit_ninja", "0006_formkitschemanode_placeholder"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="formcomponents",
            options={"ordering": ("schem", "order")},
        ),
        migrations.AddField(
            model_name="formkitschemanode",
            name="new_values",
            field=formkit_ninja.models.PydanticBaseModelField(blank=True, null=True),
        ),
    ]
